/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Copyright (c) 1997-2008 Aliasource - Groupe LINAGORA
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 * 
 *  http://www.obm.org/                                              
 * 
 * ***** END LICENSE BLOCK ***** */
const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function ObmTaskProxy() {
	this.SOAPClient = Cc["@guru4.net/soapclient;1"].getService().wrappedJSObject;

	this._listeners = [];
	this._url = null;
	this._token = {
		user: null,
		domain: null,
		session_id: null
	};
};

ObmTaskProxy.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmITaskProxy) &&
			!aIID.equals(Ci.obmIProxyBase)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},

	/*
	 * obmIProxyBase
	 */
	addListener: function(listener) {
		for (var i = 0; i < this._listeners.length; ++i) {
			if (this._listeners[i] == listener)
				return;
		}
		this._listeners.push(listener);
	},

	removeListener: function(listener) {
		for (var i = 0; i < this._listeners.length; ++i) {
			if (this._listeners[i] == listener) {
				this._listeners.splice(i, 1);
				return;
			}
		}
	},
	_notifyListeners: function(aResultCode, aMethodName, aObj) {
		// A copy is needed, a new listeners could has been added during the callbacks
		var listeners = this._listeners.slice(0, this._listeners.length);
		for (var i = 0; i < listeners.length; ++i) {
			try {
				listeners[i].onResponse(aResultCode, aMethodName, aObj, aData);
			} catch(e) {
				//Continue to notify listeners
        this.logger.error(e + " (ignored)");
			}
		}
	},

	get token() {
		return this._token;
	},
	set token(aToken) {
		this._token = aToken;
	},

	get WSDLUrl() {
		return this._url;
	},
	set WSDLUrl(aURL) {
		this._url = aURL;
	},

	/*
	 * obmITaskProxy
	 */
	_callback: function(aMethodName, aObj, aXml) {
		if (aObj && aObj.name == "Error") {
			var msg = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
			msg.data = aObj.fileName;
			this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, msg);
			return;
		}
		switch(aMethodName) {
			case "logUserIn":
				this._token.user = aObj.token[0].user[0];
				this._token.domain = aObj.token[0].domain[0];
				this._token.session_id = aObj.token[0].session_id[0];
				this._notifyListeners(Cr.NS_OK, aMethodName, null);
				break;
			case "getSync":
				this._notifyListeners(Cr.NS_OK, aMethodName, new GetSyncTaskResponseShell(aObj._return[0]));
				break;
			case "createTask":
			case "modifyTask":
			case "removeTask":
				this._notifyListeners(Cr.NS_OK, aMethodName, new TaskShell(aObj._return[0]));
				break;
			default:
			this._notifyListeners(Cr.NS_ERROR_FAILURE, aMethodName, null);
		};
	},

	getSync: function(aLastSync) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("lastsync", aLastSync);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "getSync", p, true, function(name, obj, xml) {self._callback(name, obj, xml);});
	},
	createTask: function(aTask) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("task", aTask);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "createTask", p, true, function(name, obj, xml) {self._callback(name, obj, xml);});
	},
	modifyTask: function(aTask) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("task", aTask);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "modifyTask", p, true, function(name, obj, xml) {self._callback(name, obj, xml);});
	},
	removeTask: function(aUid) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("token", this._token);
		p.add("id", aUid);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "removeTask", p, true, function(name, obj, xml) {self._callback(name, obj, xml);});
	},
	logUserIn: function(aLogin, aPwd) {
		var p = this.SOAPClient.newSOAPClientParameters();
		p.add("login", aLogin);
		p.add("password", aPwd);
		// closure
		var self = this;
		this.SOAPClient.invoke(this._url, "logUserIn", p, true, function(name, obj, xml) {self._callback(name, obj, xml);});
	}
};

function TaskShell(inner) {
	this._mInner = inner;
}

TaskShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmITask)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	get uid() {return this._mInner.uid[0]},
	get owner() {return this._mInner.owner[0]},
	get priority() {return this._mInner.priority[0]},
	get classification() {return this._mInner.classification[0]},
	get dueDate() {return this._mInner.dueDate[0]},
	get startDate() {return this._mInner.startDate[0]},
	get title() {return this._mInner.title[0]},
	get description() {return this._mInner.description[0]},
	get completionDate() {return this._mInner.completionDate[0]},
	get completion() {return this._mInner.completion[0]},
	get status() {return this._mInner.status[0]},
	get webPage() {return this._mInner.webPage[0]},
};

function GetSyncTaskResponseShell(inner) {
	this._mInner = inner;
	this._updated = [];
	this._removed = [];

	if (inner.updated) for (var i = 0; i < inner.updated.length; i++) {
		this._updated.push(new TaskShell(inner.updated[i]));
	}
	if (inner.removed) for (var i = 0; i < inner.removed.length; i++) {
		this._removed.push(inner.removed[i]);
	}
}

GetSyncTaskResponseShell.prototype = {
	QueryInterface: function(aIID) {
		if (!aIID.equals(Ci.nsISupports) &&
			!aIID.equals(Ci.obmITaskSyncResponse)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		return this;
	},
	getUpdatedTasks: function(count) {
		count.value = this._updated.length;
		return this._updated;
	},
	getRemovedTaskIds: function(count) {
		count.value = this._removed.length;
		return this._removed;
	},
	get lastsync() {
		return this._mInner.lastsync[0];
	}
};


// module

var ObmTaskProxyModule = {
	mCID: Components.ID("{fa1ceb37-dd40-4232-ab85-20ed3ac9b20c}"),
	mContractID: "@obm.org/obm-sync/taskproxy;1",
	registerSelf: function (compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(
			this.mCID,
			"OBM Task Proxy",
			this.mContractID,
			fileSpec,
			location,
			type
		);
	},
	getClassObject: function (compMgr, cid, iid) {
		if (!cid.equals(this.mCID)) {
			throw Cr.NS_ERROR_NO_INTERFACE;
		}
		if (!iid.equals(Ci.nsIFactory)) {
			throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		}
		return this.mFactory;
	},
	mFactory: {
		createInstance: function (outer, iid) {
			if (outer != null) {
				throw Cr.NS_ERROR_NO_AGGREGATION;
			}
			return (new ObmTaskProxy()).QueryInterface(iid);
		}
	},

	canUnload: function(compMgr) {
		return true;
	}
};

function NSGetModule(compMgr, fileSpec) {
	return ObmTaskProxyModule;
}

